/*
 *  TOPPERS PROJECT Home Network Working Group Software
 * 
 *  Copyright (C) 2017-2019 Cores Co., Ltd. Japan
 * 
 *  上記著作権者は，以下の(1)～(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 * 
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 * 
 *  @(#) $Id$
 */
#ifndef _MBED_API_H_
#define _MBED_API_H_

#include <time.h>
#include <stdint.h>

void mbed_api_init(void);

typedef enum {
	PIN_LED0 = 100,
	PIN_LED1,
	PIN_LED2,
	PIN_LED3,
	NC = (int)0xFFFFFFFF
} PinName;

typedef enum {
	PIN_INPUT,
	PIN_OUTPUT
} PinDirection;

typedef enum {
	PullUp = 0,
	PullDown = 3,
	PullNone = 2,
	OpenDrain = 4,
	PullDefault = PullDown
} PinMode;

typedef struct gpio_s {
	unsigned char __dummy[24];
} gpio_t;

void gpio_init(gpio_t *obj, PinName pin);
void gpio_mode(gpio_t *obj, PinMode mode);
void gpio_dir(gpio_t *obj, PinDirection direction);

void gpio_write(gpio_t *obj, int value);
int gpio_read(gpio_t *obj);

void gpio_init_in(gpio_t* gpio, PinName pin);
void gpio_init_out(gpio_t* gpio, PinName pin);

typedef struct analogin_s {
	unsigned char __dummy[24];
} analogin_t;

void analogin_init(analogin_t *obj, PinName pin);
uint16_t analogin_read_u16(analogin_t *obj);

typedef struct dac_s {
	unsigned char __dummy[24];
} dac_t;

void analogout_init(dac_t *obj, PinName pin);
void analogout_write_u16(dac_t *obj, uint16_t value);

void pin_function(PinName pin, int function);
void pin_mode(PinName pin, PinMode mode);

typedef struct pwmout_s {
	unsigned char __dummy[24];
} pwmout_t;

void pwmout_init(pwmout_t* obj, PinName pin);
void pwmout_free(pwmout_t* obj);

void pwmout_period_us(pwmout_t* obj, int us);
void pwmout_pulsewidth_us(pwmout_t* obj, int us);

void arduino_tick();

typedef enum {
    ParityNone = 0,
    ParityOdd = 1,
    ParityEven = 2,
    ParityForced1 = 3,
    ParityForced0 = 4
} SerialParity;

typedef enum {
    FlowControlNone,
    FlowControlRTS,
    FlowControlCTS,
    FlowControlRTSCTS
} FlowControl;

typedef struct serial_s {
	unsigned char __dummy[24];
} serial_t;

typedef struct spi_s {
	unsigned char __dummy[24];
} spi_t;

void serial_init(serial_t *obj, PinName tx, PinName rx);
void serial_free(serial_t *obj);
void serial_baud(serial_t *obj, int baudrate);
void serial_format(serial_t *obj, int data_bits, SerialParity parity, int stop_bits);
int serial_getc(serial_t *obj);
void serial_putc(serial_t *obj, int c);
int serial_readable(serial_t *obj);
int serial_writable(serial_t *obj);

typedef struct i2c_s {
	unsigned char __dummy[24];
} i2c_t;

void i2c_init(i2c_t *obj, PinName sda, PinName scl);
void i2c_frequency(i2c_t *obj, int hz);
int i2c_start(i2c_t *obj);
int i2c_stop(i2c_t *obj);
int i2c_read(i2c_t *obj, int address, char *data, int length, int stop);
int i2c_write(i2c_t *obj, int address, const char *data, int length, int stop);
int i2c_byte_read(i2c_t *obj, int last);
int i2c_byte_write(i2c_t *obj, int data);

void spi_init(spi_t *obj, PinName mosi, PinName miso, PinName sclk, PinName ssel);
void spi_free(spi_t *obj);
void spi_format(spi_t *obj, int bits, int mode, int slave);
void spi_frequency(spi_t *obj, int hz);
int spi_master_write(spi_t *obj, int value);

void rtc_init(void);
void rtc_free(void);
int rtc_isenabled(void);
time_t rtc_read(void);
void rtc_write(time_t t);

typedef uint32_t timestamp_t;

void us_ticker_init(void);
timestamp_t us_ticker_read(void);
void us_ticker_set_interrupt(timestamp_t timestamp);
void us_ticker_disable_interrupt(void);
void us_ticker_clear_interrupt(void);

void wait_ms(int ms);
void wait_us(int us);

#endif /* _MBED_API_H_ */
