/*
 *  TOPPERS PROJECT Home Network Working Group Software
 * 
 *  Copyright (C) 2020 Cores Co., Ltd. Japan
 * 
 *  上記著作権者は，以下の(1)～(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 * 
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 * 
 *  @(#) $Id$
 */

#include <stdint.h>
#include <sys/stat.h>
#include <errno.h>
#include <kernel.h>
#include "kernel_cfg.h"

#undef errno
extern int errno;

long __syscall_ret(unsigned long r)
{
	if (r > -4096UL) {
		errno = -r;
		return -1;
	}
	return r;
}

int _kill(int pid, int sig)
{
	return 0;
}

int _getpid(int n)
{
	return 1;
}

int _close(int fd)
{
	return __syscall_ret(shell_close(fd));
}

long _read(int fd, void *buf, long count)
{
	return __syscall_ret(shell_read(fd, buf, count));
}

long _write(int fd, const void *buf, long count)
{
	return __syscall_ret(shell_write(fd, buf, count));
}

off_t _lseek(int fd, off_t offset, int whence)
{
	off_t result;
	long ret = shell_llseek(fd, offset, &result, whence);
	return __syscall_ret(ret) ? -1 : result;
}

int _fstat(int fd, struct stat *st)
{
	struct SHELL_FILE *fp = fd_to_fp(fd);
	if (fp == NULL)
		return __syscall_ret(-EBADF);

	memset(st, 0, sizeof(*st));
	st->st_mode = S_IFCHR;
	//st->st_blksize = 1024;
	return 0;
}

// musl-1.1.18\arch\generic\bits\ioctl.h
#define TIOCGWINSZ	0x5413

struct winsize {
	unsigned short ws_row;
	unsigned short ws_col;
	unsigned short ws_xpixel;
	unsigned short ws_ypixel;
};

int _isatty(int fd)
{
	struct winsize wsz;
	long ret = shell_ioctl(fd, TIOCGWINSZ, &wsz);
	return __syscall_ret(ret);
}
